==> sqldeveloper-22.2.0.173.2018-x64.zip: oracle/dbtools/app/psr/queryUtil.js <==
function resetReport() {
    if( typeof report2File !== 'undefined' ) {
        //--print('creating file '+file);
        parser.languageServer.getLSP().createFile(outputStream, file, '', false);
    }
    totalWarnings = 0;
}

function logString(str) {
    if( typeof report2File !== 'undefined' ) {
        //if( 5< str.length() )
            //print('appending '+str.length()+' chars to file '+file);
        parser.languageServer.getLSP().documentAppend(outputStream, file, str+'\n');
    }
}


function report(node, msg) {
    totalWarnings++;
    if( 0 == cnt++ )
        logString(msg+':');
    
    if( sqlParser == null && typeof markWarnings !== 'undefined' ) {
        var send = outputStream;
        server.getLSP().createFile(send, conn.substring(1,conn.length()-1), "", false);
        server.getLSP().documentBump(send, conn);
    
        for( var i = 0; i < 20; i++ ) {
            try {
                var p = parser.getLanguageServer().getParser(conn);
                if( p != null ) {
                    sqlParser = p;
                    sqlParser.resetDiagnostics();
                    break;
                }
                java.lang.Thread.sleep(100);
            } catch( err ) {
                thisCommand.processScriptException(err);
                parser.documentBump(outputStream);
                return;
            }
        }
    }

    if( sqlParser == null && typeof markWarnings !== 'undefined' ) {
        thisCommand.processScriptException(new java.lang.Exception('sqlParser == null for '+conn));        
        parser.documentBump(outputStream);
        return;
    }
    var pos = target.src.get(tuple.get(node).from).begin; 
    var lineMap = Service.lineMap(target.input);
    var line = Service.charPos2LineNo0(lineMap,pos);
    var offset = pos - Service.lineNo2CharPos0(lineMap,line);
    if( offset < 0 )
        offset = 0;
    if( typeof lines1based !== 'undefined' )
        line++;
    if( typeof conn !== 'undefined' && conn != null )    
        logString(conn.substring(1,conn.length()-1)+'#'+line);

    var pos1 = new Position(line, offset);
    var pos2 = new Position(line, offset+3);
    var range = new Range_(pos1,pos2);
    if( typeof markWarnings !== 'undefined' ) 
        sqlParser.addDiagnostics(new Diagnostics(range,msg,Diagnostics.WARNING));
}

function ruleBrk() {
    if( 0 < cnt )
        logString('');
    cnt = 0;
}

function bumpTarget() {  
    if( typeof markWarnings === 'undefined' ) 
        return;
    if( totalWarnings == 0 ) 
        return;
    for( var i = 0; i < 20; i++ ) {
        try {
            java.lang.Thread.sleep(100);
            var p = parser.getLanguageServer().getParser(conn);
            if( p != null ) {
                p.documentBump(outputStream);
                return;
            }
        } catch( err ) {
            thisCommand.processScriptException(err);
            parser.documentBump(outputStream);
            return;
        }
    }
}



var conn; 
{ 
    if( program.targetFileName != null )
        conn = '"'+program.targetFileName+'"';
    else
        conn = parser.getRegistry().getConnString();
    if( typeof debug !== 'undefined' )    
        print('conn='+conn)    
}
//var file = conn.substring(1,conn.length()-1)+".report";
var file = parser.languageServer.rootUri + "/SqlPatroller.report";

var sqlParser = null;

var Diagnostics = Java.type('oracle.dbtools.lsp.features.Diagnostics');
var Range_ = Java.type('oracle.dbtools.lsp.features.Range');
var Position = Java.type('oracle.dbtools.lsp.features.Position');
var Program = Java.type('oracle.dbtools.arbori.Program');
var Service = Java.type('oracle.dbtools.util.Service');

var cnt = 0;
var totalWarnings = 0;

==> sqldeveloper-23.1.1.345.2114-x64.zip: oracle/dbtools/app/psr/queryUtil.js <==
function resetReport() {
    if( typeof report2File !== 'undefined' ) {
        //--print('creating file '+file);
        parser.languageServer.getLSP().createFile(outputStream, file, '', false);
    }
    totalWarnings = 0;
}

function logString(str) {
    if( typeof report2File !== 'undefined' ) {
        //if( 5< str.length() )
            //print('appending '+str.length()+' chars to file '+file);
        parser.languageServer.getLSP().documentAppend(file, str+'\n');
    }
}


function report(node, msg) {
    totalWarnings++;
    if( 0 == cnt++ )
        logString(msg+':');
    
    if( sqlParser == null && typeof markWarnings !== 'undefined' ) {
        var send = outputStream;
        server.getLSP().createFile(send, conn.substring(1,conn.length()-1), "", false);
        server.getLSP().documentBump(send, conn);
    
        for( var i = 0; i < 20; i++ ) {
            try {
                var p = parser.getLanguageServer().getParser(conn);
                if( p != null ) {
                    sqlParser = p;
                    sqlParser.resetDiagnostics();
                    break;
                }
                java.lang.Thread.sleep(100);
            } catch( err ) {
                thisCommand.processScriptException(err);
                parser.documentBump(outputStream);
                return;
            }
        }
    }

    if( sqlParser == null && typeof markWarnings !== 'undefined' ) {
        thisCommand.processScriptException(new java.lang.Exception('sqlParser == null for '+conn));        
        parser.documentBump(outputStream);
        return;
    }
    var pos = target.src.get(tuple.get(node).from).begin; 
    var lineMap = Service.lineMap(target.input);
    var line = Service.charPos2LineNo0(lineMap,pos);
    var offset = pos - Service.lineNo2CharPos0(lineMap,line);
    if( offset < 0 )
        offset = 0;
    if( typeof lines1based !== 'undefined' )
        line++;
    if( typeof conn !== 'undefined' && conn != null )    
        logString(conn.substring(1,conn.length()-1)+'#'+line);

    var pos1 = new Position(line, offset);
    var pos2 = new Position(line, offset+3);
    var range = new Range_(pos1,pos2);
    if( typeof markWarnings !== 'undefined' ) 
        sqlParser.addDiagnostics(new Diagnostics(range,msg,Diagnostics.WARNING));
}

function ruleBrk() {
    if( 0 < cnt )
        logString('');
    cnt = 0;
}

function bumpTarget() {  
    if( typeof markWarnings === 'undefined' ) 
        return;
    if( totalWarnings == 0 ) 
        return;
    for( var i = 0; i < 20; i++ ) {
        try {
            java.lang.Thread.sleep(100);
            var p = parser.getLanguageServer().getParser(conn);
            if( p != null ) {
                p.documentBump(outputStream);
                return;
            }
        } catch( err ) {
            thisCommand.processScriptException(err);
            parser.documentBump(outputStream);
            return;
        }
    }
}



var conn; 
{ 
    if( program.targetFileName != null )
        conn = '"'+program.targetFileName+'"';
    else
        conn = parser.getRegistry().getConnString();
    if( typeof debug !== 'undefined' )    
        print('conn='+conn)    
}
//var file = conn.substring(1,conn.length()-1)+".report";
var file = parser.languageServer.rootUri + "/SqlPatroller.report";

var sqlParser = null;

var Diagnostics = Java.type('oracle.dbtools.lsp.features.Diagnostics');
var Range_ = Java.type('oracle.dbtools.lsp.features.Range');
var Position = Java.type('oracle.dbtools.lsp.features.Position');
var Program = Java.type('oracle.dbtools.arbori.Program');
var Service = Java.type('oracle.dbtools.util.Service');

var cnt = 0;
var totalWarnings = 0;
